<?php

namespace JUM\PaymentProviders\Payment;

use XF\Entity\{PaymentProfile, PurchaseRequest};
use XF\Mvc\Controller;
use XF\Payment\{AbstractProvider, CallbackState};
use XF\Purchasable\Purchase;

class FreeKassa extends AbstractProvider
{
	protected $supportedCurrencies = ['RUB', 'EUR', 'USD'];

	/**
	 * @return string
	 */
	public function getTitle()
	{
		return 'Free-Kassa';
	}

	/**
	 * @param array $options
	 * @param array $errors
	 * @return bool
	 */
	public function verifyConfig(array &$options, &$errors = [])
	{
		if (!$options['merchant_id'])
		{
			$errors[] = \XF::phrase('jum_payment_providers_freekassa_merchant_id_not_set');

			return false;
		}

		if (!$options['secret_word_1'])
		{
			$errors[] = \XF::phrase('jum_payment_providers_freekassa_secret_word_1_not_set');

			return false;
		}

		if (!$options['secret_word_2'])
		{
			$errors[] = \XF::phrase('jum_payment_providers_freekassa_secret_word_2_not_set');

			return false;
		}

		return true;
	}

	/**
	 * @param PurchaseRequest $purchaseRequest
	 * @param Purchase $purchase
	 * @return array
	 */
	protected function getPaymentParams(PurchaseRequest $purchaseRequest, Purchase $purchase)
	{
		$paymentProfile = $purchase->paymentProfile;
		$options = $paymentProfile->options;
		$purchaser = $purchase->purchaser;
		$orderId = $purchaseRequest->purchase_request_id . '-' . $purchaseRequest->user_id;

		$signature = implode(':',
			[
				$options['merchant_id'],
				$purchase->cost,
				$options['secret_word_1'],
				$orderId
			]);

		$params = [
			'paymentProfile'	=>	$paymentProfile,
			'purchase'			=>	$purchase,
			'formData'			=>	[
				'm'		=>	$options['merchant_id'],
				'oa'	=>	$purchase->cost,
				'o'		=>	$orderId,
				's'		=>	md5($signature),
				'em'	=>	$purchaser->email ?? '',
				'us_rk'	=>	$purchaseRequest->request_key
			]
		];

		return $params;
	}

	public function initiatePayment(Controller $controller, PurchaseRequest $purchaseRequest, Purchase $purchase)
	{
		$viewParams = $this->getPaymentParams($purchaseRequest, $purchase);
		return $controller->view('XF:Purchase\FreeKassaInitiate', 'jum_payment_providers_payment_initiate_freekassa', $viewParams);
	}

	/**
	 * @param \XF\Http\Request $request
	 *
	 * @return CallbackState
	 */
	public function setupCallback(\XF\Http\Request $request)
	{
		$state = new CallbackState();

		$state->transactionId = $request->filter('intid', 'str');
		$state->subscriberId = $request->filter('P_EMAIL', 'str');
		$state->requestKey = $request->filter('us_rk', 'str');

		$state->orderId = $request->filter('MERCHANT_ORDER_ID', 'str');
		$state->signature = $request->filter('SIGN', 'str');
		$state->cost = $request->filter('AMOUNT', 'str');
		$state->merchantId = $request->filter('MERCHANT_ID', 'str');

		$state->ip = $request->getIp();
		$state->_POST = $_POST;

		return $state;
	}

	/**
	 * @param CallbackState $state
	 * @return bool
	 */
	public function validateTransaction(CallbackState $state)
	{
		if (!$state->transactionId || !$state->subscriberId)
		{
			$state->logType = 'error';
			$state->logMessage = 'No transaction or Subscriber ID. No action to take.';

			return false;
		}

		return parent::validateTransaction($state);
	}

	/**
	 * @param CallbackState $state
	 * @return bool
	 */
	public function validatePurchaseRequest(CallbackState $state)
	{
		$purchaseRequest = $state->getPurchaseRequest();
		$paymentProfile = $state->getPaymentProfile();
		$options = $paymentProfile->options;

		if (!$state->requestKey || !$purchaseRequest)
		{
			$state->logType = 'error';
			$state->logMessage = 'Notification does not contain a recognised purchase request.';

			return false;
		}

		$signature = implode(':',
			[
				$options['merchant_id'],
				$state->cost,
				$options['secret_word_2'],
				$state->orderId
			]);

		if (md5($signature) !== $state->signature)
		{
			$state->logType = 'error';
			$state->logMessage = 'Not verify signature';

			return false;
		}

		return parent::validatePurchaseRequest($state);
	}

	/**
	 * @param CallbackState $state
	 * @return bool
	 */
	public function validateCost(CallbackState $state)
	{
		$purchaseRequest = $state->getPurchaseRequest();

		$costValidated = $state->cost == $purchaseRequest->cost_amount;

		if (!$costValidated)
		{
			$state->logType = 'error';
			$state->logMessage = 'Invalid cost amount';

			return false;
		}

		return true;
	}

	/**
	 * @param CallbackState $state
	 */
	public function getPaymentResult(CallbackState $state)
	{
		$state->paymentResult = CallbackState::PAYMENT_RECEIVED;
	}

	/**
	 * @param CallbackState $state
	 */
	public function prepareLogData(CallbackState $state)
	{
		$state->_POST['callback_ip'] = $state->ip;
		$state->logDetails = $state->_POST ?: [];
	}

	/**
	 * @param PaymentProfile $paymentProfile
	 * @param $currencyCode
	 * @return bool
	 */
	public function verifyCurrency(PaymentProfile $paymentProfile, $currencyCode)
	{
		return (in_array($currencyCode, $this->supportedCurrencies));
	}
}