<?php

namespace JUM\PaymentProviders\Payment;

use XF\Entity\{PaymentProfile, PurchaseRequest};
use XF\Mvc\Controller;
use XF\Payment\{AbstractProvider, CallbackState};
use XF\Purchasable\Purchase;

class InterKassa extends AbstractProvider
{
	protected $supportedCurrencies = ['RUB', 'UAH'];

	/**
	 * @return string
	 */
	public function getCallbackUrl()
	{
		return \XF::app()->options()->boardUrl . '/jum_payment_callback.php?_xfProvider=' . $this->providerId;
	}

	/**
	 * @return string
	 */
	public function getTitle()
	{
		return 'Interkassa';
	}

	/**
	 * @param array $options
	 * @param array $errors
	 * @return bool
	 */
	public function verifyConfig(array &$options, &$errors = [])
	{
		if (!$options['shop_id'])
		{
			$errors[] = \XF::phrase('jum_payment_providers_interkassa_shop_id_not_set');

			return false;
		}

		if (!$options['secret_key'])
		{
			$errors[] = \XF::phrase('jum_payment_providers_interkassa_secret_key_not_set');

			return false;
		}

		return true;
	}

	/**
	 * @param PurchaseRequest $purchaseRequest
	 * @param Purchase $purchase
	 * @return array
	 */
	protected function getPaymentParams(PurchaseRequest $purchaseRequest, Purchase $purchase)
	{
		$paymentProfile = $purchase->paymentProfile;
		$options = $paymentProfile->options;
		$purchaser = $purchase->purchaser;
		$orderId = $purchaseRequest->purchase_request_id . '-' . $purchaseRequest->user_id;

		$params = [
			'paymentProfile'	=>	$paymentProfile,
			'purchase'			=>	$purchase,
			'formData'			=>	[
				'ik_co_id'			=>	$options['shop_id'],
				'ik_desc'			=>	$purchase->title,
				'ik_x_rkey'			=>	$purchaseRequest->request_key,
				'ik_am'				=>	$purchase->cost,
				'ik_cli'			=>	$purchase->purchaser->email,
				'ik_pm_no'			=>	$orderId,
				'ik_suc_u'			=>	$purchase->returnUrl,
				'ik_fal_u'			=>	$purchase->cancelUrl,
				'ik_pnd_u'			=>	$purchase->cancelUrl,
				'ik_x_subid'		=>	$purchaser->email ?? '',
				'ik_cur'			=>	$purchase->currency,
				'ik_ia_u'			=>	$this->getCallbackUrl(),
				'ik_fal_m'			=>	'GET',
				'ik_pnd_m'			=>	'GET'
			]
		];

		$params['formData']['ik_sign'] = $this->getSignature($params['formData'], $options['secret_key']);

		return $params;
	}

	/**
	 * @param Controller $controller
	 * @param PurchaseRequest $purchaseRequest
	 * @param Purchase $purchase
	 * @return \XF\Mvc\Reply\View
	 */
	public function initiatePayment(Controller $controller, PurchaseRequest $purchaseRequest, Purchase $purchase)
	{
		$viewParams = $this->getPaymentParams($purchaseRequest, $purchase);
		return $controller->view('XF:Purchase\InterkassaInitiate', 'jum_payment_providers_payment_initiate_interkassa', $viewParams);
	}

	/**
	 * @param \XF\Http\Request $request
	 *
	 * @return CallbackState
	 */
	public function setupCallback(\XF\Http\Request $request)
	{
		$state = new CallbackState();

		$state->transactionId = $request->filter('ik_inv_id', 'str');
		$state->subscriberId = $request->filter('ik_x_subid', 'str');
		$state->requestKey = $request->filter('ik_x_rkey', 'str');

		$state->paymentStatus = $request->filter('ik_inv_st', 'str');
		$state->paymentNo = $request->filter('ik_pm_no', 'str');
		$state->signature = $request->filter('ik_sign', 'str');
		$state->currency = $request->filter('ik_cur', 'str');
		$state->cost = $request->filter('ik_am', 'str');

		$state->ip = $request->getIp();
		$state->_POST = $_POST;

		return $state;
	}

	/**
	 * @param CallbackState $state
	 * @return bool
	 */
	public function validateTransaction(CallbackState $state)
	{
		if (!$state->transactionId || !$state->subscriberId)
		{
			$state->logType = 'error';
			$state->logMessage = 'No transaction or Subscriber ID. No action to take.';

			return false;
		}

		return parent::validateTransaction($state);
	}

	/**
	 * @param CallbackState $state
	 * @return bool
	 */
	public function validatePurchaseRequest(CallbackState $state)
	{
		$purchaseRequest = $state->getPurchaseRequest();
		$paymentProfile = $state->getPaymentProfile();
		$options = $paymentProfile->options;

		if (!$state->requestKey || !$purchaseRequest)
		{
			$state->logType = 'error';
			$state->logMessage = 'Notification does not contain a recognised purchase request.';

			return false;
		}

		if ($this->getSignature($state->_POST, $options['secret_key'], true) !== $state->signature)
		{
			$state->logType = 'error';
			$state->logMessage = 'Not verify signature';

			return false;
		}

		return parent::validatePurchaseRequest($state);
	}

	/**
	 * @param CallbackState $state
	 * @return bool
	 */
	public function validateCost(CallbackState $state)
	{
		$purchaseRequest = $state->getPurchaseRequest();

		$costValidated = (
			$state->cost == $purchaseRequest->cost_amount
			&& $state->currency == $purchaseRequest->cost_currency
		);

		if (!$costValidated) {
			$state->logType = 'error';
			$state->logMessage = 'Invalid cost amount';

			return false;
		}

		return parent::validateCost($state);
	}

	/**
	 * @param CallbackState $state
	 */
	public function getPaymentResult(CallbackState $state)
	{
		if ($state->paymentStatus == 'success')
		{
			$state->paymentResult = CallbackState::PAYMENT_RECEIVED;
		}
	}

	/**
	 * @param CallbackState $state
	 */
	public function prepareLogData(CallbackState $state)
	{
		$state->_POST['callback_ip'] = $state->ip;
		$state->logDetails = $state->_POST ?: [];
	}

	/**
	 * @param PaymentProfile $paymentProfile
	 * @param $currencyCode
	 * @return bool
	 */
	public function verifyCurrency(PaymentProfile $paymentProfile, $currencyCode)
	{
		return (in_array($currencyCode, $this->supportedCurrencies));
	}

	/**
	 * @param $params
	 * @param $secret
	 * @param bool $unset
	 * @return string
	 */
	protected function getSignature($params, $secret, $unset = false)
	{
		ksort($params, SORT_STRING);
		if($unset){
			unset($params['ik_sign']);
		}

		$signature = base64_encode(md5(implode(':', $params) . ':' . $secret, true));

		return $signature;
	}
}