<?php

namespace JUM\PaymentProviders\Payment;

use XF\Entity\PaymentProfile;
use XF\Entity\PurchaseRequest;
use XF\Mvc\Controller;
use XF\Payment\{AbstractProvider, CallbackState};
use XF\Purchasable\Purchase;

class QiwiWallet extends AbstractProvider
{
	protected $supportedCurrencies = ['RUB'];
	protected $currencyMap = ['RUB' => '643'];

	/**
	 * @return string
	 */
	public function getTitle()
	{
		return 'QIWI Wallet';
	}

	/**
	 * @param array $options
	 * @param array $errors
	 * @return bool
	 */
	public function verifyConfig(array &$options, &$errors = [])
	{
		if (!$options['account_phone'])
		{
			$errors[] = \XF::phrase('jum_payment_providers_qiwiwallet_account_phone_not_set');

			return false;
		}

		if (!$options['auth_token'])
		{
			$errors[] = \XF::phrase('jum_payment_providers_qiwiwallet_auth_token_not_set');

			return false;
		}

		return true;
	}

	/**
	 * @param PurchaseRequest $purchaseRequest
	 * @param Purchase $purchase
	 * @return array
	 */
	protected function getPaymentParams(PurchaseRequest $purchaseRequest, Purchase $purchase)
	{
		$paymentProfile = $purchase->paymentProfile;

		$account = $paymentProfile->options['account_phone'];
		$amount = explode('.', $purchase->cost);
		$purchaseId = $purchaseRequest->purchase_request_id . '-' . $purchaseRequest->user_id;
		$comment = '#' . $purchaseId . '# ' . $purchase->title;

		$params = [
			'paymentProfile'	=>	$purchase->paymentProfile,
			'purchase'			=>	$purchase,
			'purchaseId'		=>	$purchaseId,
			'formData'			=>	[
				'amountInteger'			=>	$amount[0],
				'amountFraction'		=>	$amount[1] ?? '0',
				'currency'				=>	'643',
				'extra[\'comment\']'	=>	$comment,
				'extra[\'account\']'	=>	$account,
				'blocked[0]'			=>	'account',
				'blocked[1]'			=>	'comment',
				'blocked[2]'			=>	'sum'
			]
		];

		return $params;
	}

	/**
	 * @param Controller $controller
	 * @param PurchaseRequest $purchaseRequest
	 * @param Purchase $purchase
	 * @return \XF\Mvc\Reply\Error|\XF\Mvc\Reply\View
	 */
	public function initiatePayment(Controller $controller, PurchaseRequest $purchaseRequest, Purchase $purchase)
	{
		$paymentProfile = $purchase->paymentProfile;
		$options = $paymentProfile->options;

		try{
			$this->getHookKey($options['hook_id'], $options['auth_token']);
		}
		catch (\Exception $exception)
		{
			return $controller->error(\XF::phrase('jum_payment_providers_qiwiwallet_configuration_error'));
		}

		$viewParams = $this->getPaymentParams($purchaseRequest, $purchase);
		return $controller->view('XF:Purchase\QiwiWalletInitiate', 'jum_payment_providers_payment_initiate_qiwiwallet', $viewParams);
	}

	/**
	 * @param \XF\Http\Request $request
	 *
	 * @return CallbackState
	 */
	public function setupCallback(\XF\Http\Request $request)
	{
		$state = new CallbackState();

		$inputRaw = $request->getInputRaw();
		$state->inputRaw = $inputRaw;

		$state->data = @json_decode($inputRaw, true);

		$state->transactionId = $state->data['payment']['txnId'];

		$state->type = $state->data['payment']['type'];
		$state->status = $state->data['payment']['status'];
		$state->errorCode = $state->data['payment']['errorCode'];
		$state->account = $state->data['payment']['account'];
		$state->comment = $state->data['payment']['comment'];
		$state->sumAmount = $state->data['payment']['sum']['amount'];
		$state->sumCurrency = $state->data['payment']['sum']['currency'];
		$state->hash = $state->data['hash'];

		$state->ip = $request->getIp();

		return $state;
	}

	/**
	 * @param CallbackState $state
	 * @return bool
	 */
	public function validateCallback(CallbackState $state)
	{
		if (!preg_match('/\#[\d]+[-]{1}[\d]+\#/', $state->comment))
		{
			$state->httpCode = 200; //Third-party payment. No reaction.
			return false;
		}

		if ($state->errorCode != '0')
		{
			$state->logType = 'error';
			$state->logMessage = 'Callback responded with an error.';

			return false;
		}

		if (!$state->hash)
		{
			$state->logType = 'error';
			$state->logMessage = 'Webhook received from QIWI does not contain a Hash string.';

			return false;
		}

		return true;
	}

	/**
	 * @param CallbackState $state
	 * @return bool
	 */
	public function validateTransaction(CallbackState $state)
	{
		if (!$state->transactionId)
		{
			$state->logType = 'error';
			$state->logMessage = 'No transaction ID. No action to take.';

			return false;
		}

		switch ($state->account)
		{
			case '':
				$state->subscriberId = 'Anonymous payer. No subscriber id.';
				break;
			default:
				$state->subscriberId = $state->account;
		}

		return parent::validateTransaction($state);
	}

	/**
	 * @param CallbackState $state
	 * @return bool
	 */
	public function validatePurchaseRequest(CallbackState $state)
	{
		$purchaseRequest = null;

		if (preg_match('/\#[\d]+[-]{1}[\d]+\#/', $state->comment, $result))
		{
			$string = str_replace('#', '', $result)[0];
			$purchaseRequestId = explode('-', $string)[0];
			$purchaseRequestUserId = explode('-', $string)[1];

			$purchaseRequest = \XF::em()->findOne('XF:PurchaseRequest', [
				'purchase_request_id' =>$purchaseRequestId,
				'user_id' => $purchaseRequestUserId
			]);
		}

		if (!$purchaseRequest)
		{
			$state->logType = 'error';
			$state->logMessage = 'Notification does not contain a recognised purchase request.';

			return false;
		}

		$state->purchaseRequest = $purchaseRequest;
		$state->requestKey = $purchaseRequest->request_key;

		$paymentProfile = $state->getPaymentProfile();
		$options = $paymentProfile->options;

		$key = $this->getHookKey($options['hook_id'], $options['auth_token']);

		if (!$key)
		{
			$state->httpCode = 503;
			$state->logType = 'error';
			$state->logMessage = 'Error retrieving key. Check your provider settings.';

			$toEmail = \XF::app()->options()->contactEmailAddress;
			if ($toEmail)
			{
				$mail = \XF::app()->mailer()->newMail()->setTo($toEmail);
				$mail->setTemplate('jum_payment_providers_qiwiwallet_error_retrieving_key', [
					'paymentProfileId' => $paymentProfile->payment_profile_id
				]);
				$mail->send();
			}

			return false;
		}

		$hashParts = implode('|', [
			$state->sumCurrency,
			$state->sumAmount,
			$state->type,
			$state->account,
			$state->transactionId
		]);

		$computed = hash_hmac('sha256', $hashParts, $key);

		if ($computed !== $state->hash)
		{
			$state->logType = 'error';
			$state->logMessage = 'Webhook received from QIWI could not be verified as being valid. Computed hash does not match received hash.';

			return false;
		}

		return parent::validatePurchaseRequest($state);
	}

	/**
	 * @param CallbackState $state
	 * @return bool
	 */
	public function validateCost(CallbackState $state)
	{
		$purchaseRequest = $state->getPurchaseRequest();

		switch ($state->status)
		{
			case 'SUCCESS':
				$costValidated = (
					$state->sumAmount == $purchaseRequest->cost_amount
					&& $state->sumCurrency == $this->currencyMap[$purchaseRequest->cost_currency]
				);

				if (!$costValidated)
				{
					$state->logType = 'error';
					$state->logMessage = 'Invalid cost amount';

					return false;
				}
				break;
		}

		return true;
	}

	/**
	 * @param CallbackState $state
	 */
	public function getPaymentResult(CallbackState $state)
	{
		if ($state->status === 'SUCCESS')
		{
			$state->paymentResult = CallbackState::PAYMENT_RECEIVED;
		}
	}

	/**
	 * @param CallbackState $state
	 */
	public function prepareLogData(CallbackState $state)
	{
		$state->data['callback_ip'] = $state->ip;
		$state->logDetails = $state->data ?: [];
	}

	/**
	 * @param PaymentProfile $paymentProfile
	 * @param $currencyCode
	 * @return bool
	 */
	public function verifyCurrency(PaymentProfile $paymentProfile, $currencyCode)
	{
		return (in_array($currencyCode, $this->supportedCurrencies));
	}

	/**
	 * @param $hookId
	 * @param $authToken
	 * @return bool|string|null
	 */
	protected function getHookKey($hookId, $authToken)
	{
		$client = $this->getHttpClient();

		try
		{
			$key = \GuzzleHttp\json_decode($client->get($this->getHooksEndpoint() . $hookId .'/key',
				$this->getHeaders($authToken))->getBody()->getContents(), true);
		}
		catch (\GuzzleHttp\Exception\BadResponseException $exception)
		{
			return null;
		}

		return base64_decode($key['key']);
	}

	/**
	 * @return string
	 */
	protected function getHooksEndpoint()
	{
		return 'https://edge.qiwi.com/payment-notifier/v1/hooks/';
	}

	/**
	 * @return \GuzzleHttp\Client
	 */
	protected function getHttpClient()
	{
		$client = \XF::app()->http()->client();
		return $client;
	}

	/**
	 * @param $authToken
	 * @return array
	 */
	protected function getHeaders($authToken)
	{
		return ['headers' => [
			'Accept' => '*/*',
			'authorization' => 'Bearer ' . $authToken
		]];
	}
}