<?php

namespace JUM\PaymentProviders\Payment;

use XF\Entity\{PaymentProfile, PurchaseRequest};
use XF\Mvc\Controller;
use XF\Payment\{AbstractProvider, CallbackState};
use XF\Purchasable\Purchase;

class RoboKassa extends AbstractProvider
{
	protected $supportedCurrencies = ['RUB'];

	/**
	 * @return string
	 */
	public function getTitle()
	{
		return 'ROBOKASSA';
	}

	/**
	 * @param array $options
	 * @param array $errors
	 * @return bool
	 */
	public function verifyConfig(array &$options, &$errors = [])
	{
		if (empty($options['merchant_id']))
		{
			$errors[] = \XF::phrase('jum_payment_providers_robokassa_merchant_id_not_set');
			return false;
		}

		if (empty($options['pass_1']))
		{
			$errors[] = \XF::phrase('jum_payment_providers_robokassa_pass_1_not_set');
			return false;
		}

		if (empty($options['pass_2']))
		{
			$errors[] = \XF::phrase('jum_payment_providers_robokassa_pass_2_not_set');
			return false;
		}

		if ($errors)
		{
			return false;
		}

		return true;
	}

	/**
	 * @param PurchaseRequest $purchaseRequest
	 * @param Purchase $purchase
	 * @return array
	 */
	protected function getPaymentParams(PurchaseRequest $purchaseRequest, Purchase $purchase)
	{
		$paymentProfile = $purchase->paymentProfile;
		$options = $paymentProfile->options;
		$purchaser = $purchase->purchaser;
		$boardHost = parse_url(\XF::app()->options()->boardUrl);

		$params = [
			'paymentProfile'	=>	$paymentProfile,
			'purchase'			=>	$purchase,
			'formData'			=>	[
				'MrchLogin'			=>	$options['merchant_id'],
				'OutSum'			=>	$purchase->cost,
				'InvDesc'			=>	$purchase->purchasableTitle . ', ' . $boardHost['host'],
				'Encoding'			=>	'utf-8',
				'Email'				=>	$purchaser->email ?? '',
				'shp_rk'			=>	$purchaseRequest->request_key
			]
		];

		$params['formData']['SignatureValue'] = md5(implode(':', [
			$options ['merchant_id'],
			$purchase->cost,
			'',
			$options['pass_1'],
			'shp_rk=' .$purchaseRequest->request_key
		]));

		return $params;
	}

	/**
	 * @param Controller $controller
	 * @param PurchaseRequest $purchaseRequest
	 * @param Purchase $purchase
	 * @return \XF\Mvc\Reply\View
	 */
	public function initiatePayment(Controller $controller, PurchaseRequest $purchaseRequest, Purchase $purchase)
	{
		$viewParams = $this->getPaymentParams($purchaseRequest, $purchase);
		return $controller->view('XF:Purchase\RoboKassaInitiate', 'jum_payment_providers_payment_initiate_robokassa', $viewParams);
	}

	/**
	 * @param \XF\Http\Request $request
	 *
	 * @return CallbackState
	 */
	public function setupCallback(\XF\Http\Request $request)
	{
		$state = new CallbackState();

		$state->transactionId = $request->filter('InvId', 'str');
		$state->subscriberId = $request->filter('EMail', 'str');
		$state->requestKey = $request->filter('shp_rk', 'str');

		$state->signature = $request->filter('SignatureValue', 'str');
		$state->cost = $request->filter('OutSum', 'str');
		$state->ip = $request->getIp();

		$state->ip = $request->getIp();
		$state->_POST = $_POST;

		return $state;
	}

	/**
	 * @param CallbackState $state
	 * @return bool
	 */
	public function validateTransaction(CallbackState $state)
	{
		if (!$state->transactionId || !$state->subscriberId)
		{
			$state->logType = 'error';
			$state->logMessage = 'No transaction or Subscriber ID. No action to take.';

			return false;
		}

		return parent::validateTransaction($state);
	}

	/**
	 * @param CallbackState $state
	 * @return bool
	 */
	public function validatePurchaseRequest(CallbackState $state)
	{
		$purchaseRequest = $state->getPurchaseRequest();
		$paymentProfile = $state->getPaymentProfile();
		$options = $paymentProfile->options;

		if (!$state->requestKey || !$purchaseRequest)
		{
			$state->logType = 'error';
			$state->logMessage = 'Notification does not contain a recognised purchase request.';

			return false;
		}

		$signature = strtoupper(md5(implode(':',
			[
				$state->cost,
				$state->transactionId,
				$options['pass_2'],
				'shp_rk='.$state->requestKey
			])));

		if ($signature !== $state->signature)
		{
			$state->logType = 'error';
			$state->logMessage = 'Not verify signature';

			return false;
		}

		return parent::validatePurchaseRequest($state);
	}

	/**
	 * @param CallbackState $state
	 * @return bool
	 */
	public function validateCost(CallbackState $state)
	{
		$purchaseRequest = $state->getPurchaseRequest();

		$costValidated = $state->cost == $purchaseRequest->cost_amount;

		if (!$costValidated)
		{
			$state->logType = 'error';
			$state->logMessage = 'Invalid cost amount';

			return false;
		}

		return true;
	}

	/**
	 * @param CallbackState $state
	 */
	public function getPaymentResult(CallbackState $state)
	{
		$state->paymentResult = CallbackState::PAYMENT_RECEIVED;
	}

	/**
	 * @param CallbackState $state
	 */
	public function prepareLogData(CallbackState $state)
	{
		$state->_POST['callback_ip'] = $state->ip;
		$state->logDetails = $state->_POST ?: [];
	}

	/**
	 * @param PaymentProfile $paymentProfile
	 * @param $currencyCode
	 * @return bool
	 */
	public function verifyCurrency(PaymentProfile $paymentProfile, $currencyCode)
	{
		return (in_array($currencyCode, $this->supportedCurrencies));
	}
}