<?php

namespace JUM\PaymentProviders\Payment;

use XF\Entity\{PaymentProfile, PurchaseRequest};
use XF\Mvc\Controller;
use XF\Payment\{AbstractProvider, CallbackState};
use XF\Purchasable\Purchase;

class WebMoney extends AbstractProvider
{
	protected $supportedCurrencies = [
		'BYN', 'EUR', 'RUB',
		'UAH', 'USD'
	];

	/**
	 * @return string
	 */
	public function getTitle()
	{
		return 'WebMoney';
	}

	/**
	 * @param array $options
	 * @param array $errors
	 * @return bool
	 */
	public function verifyConfig(array &$options, &$errors = [])
	{
		if (!$options['payee_purse'])
		{
			$errors[] = \XF::phrase('jum_payment_providers_webmoney_payee_purse_not_set');

			return false;
		}

		if (!$options['secret_key'])
		{
			$errors[] = \XF::phrase('jum_payment_providers_webmoney_secret_key_not_set');

			return false;
		}

		return true;
	}

	/**
	 * @param PurchaseRequest $purchaseRequest
	 * @param Purchase $purchase
	 * @return array
	 */
	protected function getPaymentParams(PurchaseRequest $purchaseRequest, Purchase $purchase)
	{
		$paymentProfile = $purchase->paymentProfile;
		$options = $paymentProfile->options;
		$purchaser = $purchase->purchaser;

		$params = [
			'paymentProfile'	=>	$paymentProfile,
			'purchase'			=>	$purchase,
			'formData'			=>	[
				'LMI_PAYEE_PURSE'			=>	$options ['payee_purse'],
				'LMI_PAYMENT_DESC_BASE64'	=>	base64_encode($purchase->title),
				'LMI_PAYMENT_AMOUNT'		=>	$purchase->cost,
				'LMI_SUCCESS_URL'			=>	$purchase->returnUrl,
				'LMI_SUCCESS_METHOD'		=>	'1',
				'LMI_FAIL_URL'				=>	$purchase->cancelUrl,
				'LMI_FAIL_METHOD'			=>	'2',
				'LMI_PAYMER_EMAIL'			=>	$purchaser->email ?? '',
				'LMI_RESULT_URL'			=>	$this->getCallbackUrl(),
				'PURCHASE_REQUEST'			=>	$purchaseRequest->request_key
			]
		];

		return $params;
	}

	/**
	 * @param Controller $controller
	 * @param PurchaseRequest $purchaseRequest
	 * @param Purchase $purchase
	 * @return \XF\Mvc\Reply\View
	 */
	public function initiatePayment(Controller $controller, PurchaseRequest $purchaseRequest, Purchase $purchase)
	{
		$viewParams = $this->getPaymentParams($purchaseRequest, $purchase);
		return $controller->view('XF:Purchase\WebMoneyInitiate', 'jum_payment_providers_payment_initiate_webmoney', $viewParams);
	}

	/**
	 * @param \XF\Http\Request $request
	 *
	 * @return CallbackState
	 */
	public function setupCallback(\XF\Http\Request $request)
	{
		$state = new CallbackState();

		$state->transactionId = $request->filter('LMI_SYS_TRANS_NO', 'str');
		$state->requestKey = $request->filter('PURCHASE_REQUEST', 'str');
		$state->subscriberId = $request->filter('LMI_PAYER_WM', 'str');

		$state->lmiHash = $request->filter('LMI_HASH', 'str');
		$state->lmiMode = $request->filter('LMI_MODE', 'str');
		$state->lmiPaymentAmount = $request->filter('LMI_PAYMENT_AMOUNT', 'str');
		$state->lmiPayeePurse = $request->filter('LMI_PAYEE_PURSE', 'str');
		$state->lmiPaymentNo = $request->filter('LMI_PAYMENT_NO', 'str');
		$state->lmiPayerWm = $request->filter('LMI_PAYER_WM', 'str');
		$state->lmiPayerPurse = $request->filter('LMI_PAYER_PURSE', 'str');
		$state->lmiSysInvsNo = $request->filter('LMI_SYS_INVS_NO', 'str');
		$state->lmiSysTransDate = $request->filter('LMI_SYS_TRANS_DATE', 'str');
		$state->lmiSdpType = $request->filter('LMI_SDP_TYPE', 'str');
		$state->lmiHash2 = $request->filter('LMI_HASH2', 'str');

		$state->ip = $request->getIp();
		$state->_POST = $_POST;

		return $state;
	}

	/**
	 * @param CallbackState $state
	 * @return bool
	 */
	public function validateTransaction(CallbackState $state)
	{
		if (empty($state->_POST))
		{
			return false; //Preliminary inquiry merchant. We don't log this.
		}

		if (!$state->transactionId || !$state->subscriberId)
		{
			$state->logType = 'error';
			$state->logMessage = 'No transaction or Subscriber ID. No action to take.';

			return false;
		}

		return parent::validateTransaction($state);
	}

	/**
	 * @param CallbackState $state
	 * @return bool
	 */
	public function validatePurchaseRequest(CallbackState $state)
	{
		$purchaseRequest = $state->getPurchaseRequest();

		if (!$state->requestKey || !$purchaseRequest)
		{
			$state->logType = 'error';
			$state->logMessage = 'Notification does not contain a recognised purchase request.';

			return false;
		}

		if ($this->getInSignature($state) !== $state->lmiHash2)
		{
			$state->logType = 'error';
			$state->logMessage = 'Not verify signature';

			return false;
		}

		return parent::validatePurchaseRequest($state);
	}

	/**
	 * @param CallbackState $state
	 * @return bool
	 */
	public function validateCost(CallbackState $state)
	{
		$purchaseRequest = $state->getPurchaseRequest();

		$costValidated = ($state->lmiPaymentAmount == $purchaseRequest->cost_amount);

		if (!$costValidated)
		{
			$state->logType = 'error';
			$state->logMessage = 'Invalid cost amount';

			return false;
		}

		return true;
	}

	/**
	 * @param CallbackState $state
	 */
	public function getPaymentResult(CallbackState $state)
	{
		switch ($state->lmiSdpType)
		{
			case '0':
				$state->subscriberId = 'No subscriber id. Payment by money transfer system';
				break;
			case '3':
				$state->subscriberId = 'No subscriber id. Payment by Alfa-click.';
				break;
			case '4':
				$state->subscriberId = 'No subscriber id. Payment by cards of Russian banks.';
				break;
			case '5':
				$state->subscriberId = 'No subscriber id. Payment by Internet banking Russian standard.';
				break;
			case '6':
				$state->subscriberId = 'No subscriber id. Payment by Internet banking VTB24.';
				break;
			case '8':
				$state->subscriberId = 'No subscriber id. Payment by terminals and banks (only for WMU-wallets)';
				break;
			case '9':
				$state->subscriberId = 'No subscriber id. Payment by Internet banking Promsvyazbank.';
				break;
			case '10':
				$state->subscriberId = 'No subscriber id. Payment by Visa\MasterCard\NSMEP\Privat24 cards (only for WMU-wallets).';
				break;
			case '11':
				$state->subscriberId = 'No subscriber id. Payment by mobile Commerce MTS.';
				break;
			case '12':
				$state->subscriberId = 'No subscriber id. Payment by mobile Commerce Beeline.';
				break;
			case '14':
				$state->subscriberId = 'No subscriber id. Payment by Sberbank Online.';
				break;
			case '19':
				$state->subscriberId = 'No subscriber id. Payment by Bitcoin.';
				break;
		}

		$state->paymentResult = CallbackState::PAYMENT_RECEIVED;
	}

	/**
	 * @param CallbackState $state
	 */
	public function prepareLogData(CallbackState $state)
	{
		$state->_POST['callback_ip'] = $state->ip;
		$state->logDetails = $state->_POST ?: [];
	}

	public function verifyCurrency(PaymentProfile $paymentProfile, $currencyCode)
	{
		return (in_array($currencyCode, $this->supportedCurrencies));
	}

	protected function getInSignature(CallbackState $state)
	{
		$paymentProfile = $state->getPaymentProfile();
		$options = $paymentProfile->options;

		$getSign = $options['secret_key'];

		$hashRequest =
			$state->lmiPayeePurse.';'.
			$state->lmiPaymentAmount.';'.
			$state->lmiPaymentNo.';'.
			$state->lmiMode.';'.
			$state->lmiSysInvsNo.';'.
			$state->transactionId.';'.
			$state->lmiSysTransDate.';'.
			$getSign.';'.
			$state->lmiPayerPurse.';'.
			$state->lmiPayerWm
		;

		$signature = strtoupper(hash("sha256", $hashRequest));

		return $signature;
	}
}