<?php

namespace JUM\PaymentProviders\Payment;

use XF\Entity\{PaymentProfile, PurchaseRequest};
use XF\Mvc\Controller;
use XF\Payment\{AbstractProvider, CallbackState};
use XF\Purchasable\Purchase;

class YooMoney extends AbstractProvider
{
	protected $supportedCurrencies = ['RUB'];
	protected $currencyMap = ['RUB' => '643'];

	/**
	 * @return string
	 */
	public function getTitle()
	{
		return 'YooMoney';
	}

	/**
	 * @param array $options
	 * @param array $errors
	 * @return bool
	 */
	public function verifyConfig(array &$options, &$errors = [])
	{
		if (!$options['wallet_number'])
		{
			$errors[] = \XF::phrase('jum_payment_providers_yoomoney_wallet_number_not_set');

			return false;
		}

		if (!$options['notice_secret'])
		{
			$errors[] = \XF::phrase('jum_payment_providers_yoomoney_notice_secret_not_set');

			return false;
		}

		return true;
	}

	/**
	 * @param PurchaseRequest $purchaseRequest
	 * @param Purchase $purchase
	 * @return array
	 */
	protected function getPaymentParams(PurchaseRequest $purchaseRequest, Purchase $purchase)
	{
		$paymentProfile = $purchase->paymentProfile;

		$params = [
			'paymentProfile'	=>	$paymentProfile,
			'purchase'			=>	$purchase,
			'formData'			=>	[
				'receiver'			=>	$paymentProfile->options['wallet_number'],
				'quickpay-form'		=>	'shop',
				'targets'			=>	$purchase->title,
				'paymentType'		=>	$paymentProfile->options['payment_type'],
				'sum'				=>	$purchase->cost,
				'formcomment'		=>	$purchase->title,
				'label'				=>	$purchaseRequest->request_key,
				'successURL'		=>	$purchase->returnUrl
			]
		];

		if ($paymentProfile->options['payment_type'] === 'user')
		{
			unset($params['formData']['paymentType']);
		}

		return $params;
	}

	/**
	 * @param Controller $controller
	 * @param PurchaseRequest $purchaseRequest
	 * @param Purchase $purchase
	 * @return \XF\Mvc\Reply\View
	 */
	public function initiatePayment(Controller $controller, PurchaseRequest $purchaseRequest, Purchase $purchase)
	{
		$viewParams = $this->getPaymentParams($purchaseRequest, $purchase);
		return $controller->view('XF:Purchase\YooMoneyInitiate', 'jum_payment_providers_payment_initiate_yoomoney', $viewParams);
	}

	/**
	 * @param \XF\Http\Request $request
	 *
	 * @return CallbackState
	 */
	public function setupCallback(\XF\Http\Request $request)
	{
		$state = new CallbackState();

		$state->subscriberId = $request->filter('sender', 'str');
		$state->transactionId = $request->filter('operation_id', 'str');
		$state->requestKey = $request->filter('label', 'str');

		$state->notificationType = $request->filter('notification_type', 'str');
		$state->amount = $request->filter('amount', 'str');
		$state->datetime = $request->filter('datetime', 'str');
		$state->codpro = $request->filter('codepro', 'str');
		$state->withdrawAmount = $request->filter('withdraw_amount', 'str');
		$state->sha1Hash = $request->filter('sha1_hash', 'str');
		$state->unaccepted = $request->filter('unaccepted', 'str');
		$state->currency = $request->filter('currency', 'str');

		$state->ip = $request->getIp();
		$state->_POST = $_POST;

		return $state;
	}

	/**
	 * @param CallbackState $state
	 * @return bool
	 */
	public function validatePurchaseRequest(CallbackState $state)
	{
		$purchaseRequest = $state->getPurchaseRequest();
		$paymentProfile = $state->getPaymentProfile();
		// TODO: проверить ошибку при тестовом запросе
		$options = $paymentProfile->options;

		if (!$state->requestKey || !$purchaseRequest)
		{
			$state->logType = 'error';
			$state->logMessage = 'Notification does not contain a recognised purchase request.';

			return false;
		}

		$sha1Hash = sha1(implode('&', [
			$state->notificationType,
			$state->transactionId,
			$state->amount,
			$state->currency,
			$state->datetime,
			$state->subscriberId,
			$state->codpro,
			$options['notice_secret'],
			$state->requestKey
		]));

		if ($sha1Hash !== $state->sha1Hash)
		{
			$state->logType = 'error';
			$state->logMessage = 'Not verify signature';

			return false;
		}

		return parent::validatePurchaseRequest($state);
	}

	/**
	 * @param CallbackState $state
	 * @return bool
	 */
	public function validateCost(CallbackState $state)
	{
		$purchaseRequest = $state->getPurchaseRequest();

		switch ($state->unaccepted)
		{
			case 'false':
				$costValidated = (
					$state->withdrawAmount == $purchaseRequest->cost_amount
					&& $state->currency == $this->currencyMap[$purchaseRequest->cost_currency]
				);

				if (!$costValidated)
				{
					$state->logType = 'error';
					$state->logMessage = 'Invalid cost amount';

					return false;
				}
				break;
		}

		return true;
	}

	/**
	 * @param CallbackState $state
	 */
	public function getPaymentResult(CallbackState $state)
	{
		switch ($state->notificationType)
		{
			case 'card-incoming':

				$state->subscriberId = 'Payment by card. No subscriber id.';
				break;
		}

		if ($state->unaccepted === 'false' && $state->codpro === 'false')
		{
			$state->paymentResult = CallbackState::PAYMENT_RECEIVED;
		}
	}

	/**
	 * @param CallbackState $state
	 */
	public function prepareLogData(CallbackState $state)
	{
		$state->_POST['callback_ip'] = $state->ip;
		$state->logDetails = $state->_POST ?: [];
	}

	/**
	 * @param PaymentProfile $paymentProfile
	 * @param $currencyCode
	 * @return bool
	 */
	public function verifyCurrency(PaymentProfile $paymentProfile, $currencyCode)
	{
		return (in_array($currencyCode, $this->supportedCurrencies));
	}
}