<?php

namespace JUM\PaymentProviders\XF\Admin\Controller;

use XF\Mvc\FormAction;

class PaymentProfile extends XFCP_PaymentProfile
{
	/**
	 * @param \XF\Entity\PaymentProfile $profile
	 * @param \XF\Entity\PaymentProvider $provider
	 * @return \XF\Mvc\FormAction
	 * @throws \XF\Mvc\Reply\Exception
	 */
	protected function profileSaveProcess(\XF\Entity\PaymentProfile $profile, \XF\Entity\PaymentProvider $provider)
	{
		if ($provider->provider_id != 'qiwiwallet') return parent::profileSaveProcess($profile, $provider);

		$form = $this->formAction();

		$input = $this->filter([
			'provider_id' => 'str',
			'title' => 'str',
			'display_title' => 'str',
			'options' => 'array-str'
		]);

		$authToken = $input['options']['auth_token'];
		$hookId = $this->setupWebHookId($authToken);

		$input['options']['hook_id'] = $hookId;

		$form->validate(function(FormAction $form) use ($provider, $input)
		{
			$provider->getHandler()->verifyConfig($input['options'], $errors);
			if ($errors)
			{
				$form->logErrors($errors);
			}
		});

		$form->basicEntitySave($profile, $input);

		return $form;
	}

	/**
	 * @param $authToken
	 * @return bool
	 * @throws \XF\Mvc\Reply\Exception
	 */
	protected function setupWebHookId($authToken)
	{
		try{
			if (!$authToken)
			{
				return false;
			}

			$currentHookId = $this->getActiveHook($authToken)['hookId'];
		}
		catch (\GuzzleHttp\Exception\RequestException $e)
		{
			switch ($e->getCode())
			{
				case 400:
					throw $this->exception($this->error('Request syntax error (incorrect data format).'));
					break;
				case 401:
					throw $this->exception($this->error('Invalid token or token expired.'));
					break;
				case 403:
					throw $this->exception($this->error('No rights to this request (token does not have enough permissions).'));
					break;
				case 404:
					return $this->registerNewHook($authToken)['hookId'];
					break;
				case 422:
					throw $this->exception($this->error('Incorrect parameters of the hook.'));
					break;
				case 423:
					throw $this->exception($this->error('Too many requests, service temporarily unavailable.'));
					break;
				case 500:
					throw $this->exception($this->error('Internal service error (URL length exceeded).'));
					break;
				default:
					throw $this->exception($this->error('Unknown error.'));
			}
		}

		$this->deleteOldHook($authToken, $currentHookId);

		return $this->registerNewHook($authToken)['hookId'];
	}

	/**
	 * @param $authToken
	 * @return mixed
	 */
	protected function getActiveHook($authToken)
	{
		$client = $this->getHttpClient();

		$hookId = \GuzzleHttp\json_decode($client->get($this->getHooksEndpoint() . 'active',
			$this->getHeaders($authToken))->getBody()->getContents(), true);

		return $hookId;
	}

	/**
	 * @param $authToken
	 * @param $hookId
	 */
	protected function deleteOldHook($authToken, $hookId)
	{
		$client = $this->getHttpClient();

		$client->delete($this->getHooksEndpoint() . $hookId, $this->getHeaders($authToken));
	}

	/**
	 * @param $authToken
	 * @return mixed
	 */
	protected function registerNewHook($authToken)
	{
		$client = $this->getHttpClient();

		$hookId = \GuzzleHttp\json_decode($client->put($this->getHooksEndpoint(), [
				'query' => [
					'hookType' => '1',
					'param' => $this->getCallbackUrl(),
					'txnType' => '0'
				]
			] + $this->getHeaders($authToken))->getBody()->getContents(), true);

		return $hookId;
	}

	/**
	 * @param $hookId
	 * @param $authToken
	 * @return mixed
	 */
	protected function getHookKey($hookId, $authToken)
	{
		$client = $this->getHttpClient();

		$key = \GuzzleHttp\json_decode($client->get($this->getHooksEndpoint() . $hookId .'/key',
			$this->getHeaders($authToken))->getBody()->getContents(), true);

		return $key['key'];
	}

	/**
	 * @return string
	 */
	protected function getHooksEndpoint()
	{
		return 'https://edge.qiwi.com/payment-notifier/v1/hooks/';
	}

	/**
	 * @return \GuzzleHttp\Client
	 */
	protected function getHttpClient()
	{
		$client = \XF::app()->http()->client();
		return $client;
	}

	protected function getHeaders($authToken)
	{
		return ['headers' => [
			'Accept' => '*/*',
			'authorization' => 'Bearer ' . $authToken
		]];
	}

	/**
	 * @return string
	 */
	protected function getCallbackUrl()
	{
		return \XF::app()->options()->boardUrl . '/jum_payment_callback.php?_xfProvider=qiwiwallet';
	}
}